".First.lib" <-
function (lib, pkg) 
{
    library(kftrack)
    library(date)
    library(locfit, warn = FALSE)
    cat("\n\n")
    cat(paste(rep("=", options()$width), collapse = ""), "\n\n")
    cat("Welcome to KFSST - A package for improving tracking of tagged\n")
    cat("sea creatures by including Sea Surface Temperature (SST)\n")
    cat("in the Kalman Filter (KF) based reconstruction model. \n\n")
    cat("What you typically want to do is the following sequence:\n\n")
    cat("1) Read in one or more track(s)\n")
    cat("2) Obtain a corresponding SST-field\n")
    cat("3) Smooth the SST-field and save it in suitable format\n")
    cat("4) Reconstruct the track(s)\n")
    cat("5) Investigate the fitted track by plots and summaries\n\n")
    cat("A slightly more detailed description of these five steps\n")
    cat("including what functions may be useful and how to call them is\n")
    cat("available via the function 'road.map'. If needed simply type: \n\n")
    cat("   road.map()\n\n")
    cat("and the text will be displayed.\n\n")
    cat("Check also the complete example in the 'blue.shark' dataset\n")
    cat("documentation by typing:\n\n")
    cat("  example(blue.shark)\n\n")
    cat("Thank you for your efforts in downloading and trying this\n")
    cat("software. If you encounter any bugs or have any questions -\n")
    cat("please don't hesitate to contact me:\n\n")
    cat("Anders Nielsen <anders.nielsen@hawaii.edu>\n\n")
    if (.Platform$OS.type == "windows") {
        cat(paste(rep("=", options()$width), collapse = ""), 
            "\n\n")
        cat("WINDOWS USERS: Please press (Ctrl+W) or use the menu \n")
        cat("(Misc->Buffered output) to switch the delayed output off.\n\n")
    }
    cat(paste(rep("=", options()$width), collapse = ""), "\n\n")
}
".generate.dat.file.kfsst" <-
function (data, file = "kfsst.dat", fix.first = TRUE, fix.last = TRUE, 
    theta.active = c(u.active, v.active, D.active, bx.active, 
        by.active, bsst.active, sx.active, sy.active, ssst.active, 
        a0.active, b0.active), theta.init = c(u.init, v.init, 
        D.init, bx.init, by.init, bsst.init, sx.init, sy.init, 
        ssst.init, a0.init, b0.init), u.active = TRUE, v.active = TRUE, 
    D.active = TRUE, bx.active = TRUE, by.active = TRUE, bsst.active = TRUE, 
    sx.active = TRUE, sy.active = TRUE, ssst.active = TRUE, a0.active = TRUE, 
    b0.active = TRUE, u.init = 0, v.init = 0, D.init = 100, bx.init = 0, 
    by.init = 0, bsst.init = 0, sx.init = 0.5, sy.init = 1.5, 
    ssst.init = 0.1, a0.init = 0.001, b0.init = 0, var.struct = "solstice", 
    dev.pen = 0) 
{
    "%+%" <- function(s1, s2) paste(s1, s2, sep = "")
    tostr <- function(x) paste(as.numeric(x), collapse = "\t")
    var.flags <- switch(var.struct, uniform = c(0, 0, 0), solstice = c(1, 
        0, 0), daily = c(0, 1, dev.pen), warning("No matching variance structure found"))
    header <- "#Auto generated data file from R-KFsst \n#" %+% 
        date() %+% "\n#\n" %+% "# Number of data points\n  " %+% 
        tostr(nrow(data)) %+% "\n#\n" %+% "# 1 if first point is true release position; 0 otherwise\n  " %+% 
        tostr(fix.first) %+% "\n#\n" %+% "# 1 if last point is true recapture position; 0 otherwise\n  " %+% 
        tostr(fix.last) %+% "\n#\n" %+% "# active parameters \n# u\tv\tD\tbx\tby\tbsst\tsx\tsy\tssst\ta0\tb0\n  " %+% 
        tostr(theta.active) %+% "\n#\n" %+% "# initial values \n# u\tv\tD\tbx\tby\tbsst\tsx\tsy\tssst\ta0\tb0\n  " %+% 
        tostr(theta.init) %+% "\n#\n" %+% "# latitude errors \n# cos\tdev\tdeviation penalty weight\n  " %+% 
        tostr(var.flags) %+% "\n#\n" %+% "# Positions \n# day\tmonth\tyear\tlong\tlati\tsst\n  "
    cat(header, file = file)
    write.table(data, file = file, row.names = FALSE, col.names = FALSE, 
        sep = "\t", eol = "\n  ", append = TRUE)
    cat("\n", file = file, append = TRUE)
    class(header) <- "kfssthead"
    return(header)
}
".myflush" <-
function () 
if (.Platform$OS.type == "windows") {
    flush.console()
} else {
}
".plotsst" <-
function (x, ci = FALSE, points = TRUE, pred = TRUE, most = TRUE, 
    ...) 
{
    mypch <- c(ifelse(x$fix.first, 6, 4), rep(4, x$nobs - 2), 
        ifelse(x$fix.last, 2, 4))
    yrange <- .addrange(c(if (points) {
        x$SST[, "o"]
    }, if (pred) {
        x$SST[-1, "p"]
    }, if (most) {
        x$SST[, "smooth"]
    }))
    plot(x$days.at.liberty, x$SST[, "o"], ylab = "SST", pch = mypch, 
        ylim = yrange, las = 3, xlab = "", axes = FALSE, type = ifelse(points, 
            "b", "n"), col = gray(0.9))
    if (points) {
        points(x$days.at.liberty, x$SST[, "o"], pch = mypch)
    }
    if (pred) {
        lines(x$days.at.liberty[-1], x$SST[-1, "p"], col = "green")
    }
    if (most) {
        lines(x$days.at.liberty, x$SST[, "smooth"], col = "blue")
    }
}
".read.output.kfsst" <-
function (data, fix.first = TRUE, fix.last = TRUE, theta.active = c(u.active, 
    v.active, D.active, bx.active, by.active, sx.active, sy.active, 
    a0.active, b0.active), theta.init = c(u.init, v.init, D.init, 
    bx.init, by.init, sx.init, sy.init, a0.init, b0.init), u.active = TRUE, 
    v.active = TRUE, D.active = TRUE, bx.active = TRUE, by.active = TRUE, 
    sx.active = TRUE, sy.active = TRUE, a0.active = TRUE, b0.active = TRUE, 
    u.init = 0, v.init = 0, D.init = 100, bx.init = 0, by.init = 0, 
    sx.init = 0.5, sy.init = 1.5, a0.init = 0.001, b0.init = 0, 
    var.struct = "solstice", dev.pen = 0) 
{
    getpar <- function(what, file) {
        txt <- readLines(file)
        return(as.numeric(strsplit(txt[grep(what, txt) + 1], 
            split = " ")[[1]]))
    }
    getrep <- function(what, file) {
        txt <- readLines(file)
        return(as.numeric(strsplit(txt[grep(what, txt)], split = " ")[[1]][3]))
    }
    theta.names <- c("u", "v", "D", "bx", "by", "bsst", "sx", 
        "sy", "ssst")
    if (var.struct == "solstice") {
        theta.names <- c(theta.names, "a0", "b0")
    }
    if (file.access("kfsst.par") != 0) {
        warning("File \"kfsst.par\" not found (possibly because there was no solution to minimization problem)", 
            call. = FALSE)
        npar <- NA
        nlogL <- NA
        max.grad.comp <- NA
        estimates <- NA
    }
    else {
        tmp <- as.numeric(scan("kfsst.par", what = "character", 
            16, quiet = TRUE)[c(6, 11, 16)])
        npar <- tmp[1]
        nlogL <- tmp[2]
        max.grad.comp <- tmp[3]
        estimates <- sapply(c("uu", "vv", "D", "bx", "by", "bsst", 
            "vx", "vy", "vsst"), getpar, file = "kfsst.par")
        if (var.struct == "solstice") {
            estimates <- c(estimates, sapply(c("a0", "b0"), getpar, 
                file = "kfsst.par"))
        }
        names(estimates) <- theta.names
    }
    if (file.access("kfsst.rep") != 0) {
        warning("File \"kfsst.rep\" not found", call. = FALSE)
        spd <- NA
        hdg <- NA
    }
    else {
        spd <- getrep("spd", "kfsst.rep")
        hdg <- getrep("hdg", "kfsst.rep")
    }
    if (file.access("kfsst.std") != 0) {
        warning("File \"kfsst.std\" not found (possibly the hessian was not estimated)", 
            call. = FALSE)
        std.dev <- NA
    }
    else {
        dat <- read.table("kfsst.std", skip = 1)
        tmp <- dat[dat[, 2] %in% c("sduu", "sdvv", "sdD", "sdbx", 
            "sdby", "sdbsst", "sdvx", "sdvy", "sdvsst"), 3:4]
        if (var.struct == "solstice") {
            if (theta.active[10]) {
                tmp <- rbind(tmp, dat[dat[, 2] == "a0", 3:4])
            }
            else {
                tmp <- rbind(tmp, c(0, 0))
            }
            if (theta.active[11]) {
                tmp <- rbind(tmp, dat[dat[, 2] == "b0", 3:4])
            }
            else {
                tmp <- rbind(tmp, c(0, 0))
            }
        }
        std.dev <- tmp[, 2]
        names(std.dev) <- paste("sd.", theta.names, sep = "")
    }
    mptfilename <- paste("mpt_", paste(as.numeric(c(theta.active[c(1:5, 
        7:8)], var.struct == "solstice", var.struct == "daily")), 
        collapse = ""), ".dat", sep = "")
    if (file.access(mptfilename) != 0) {
        warning("File not found")
    }
    else {
        tmp <- read.table(mptfilename, skip = 3, header = FALSE)
        name <- strsplit(readLines(mptfilename, 3)[3], split = " ")[[1]]
        colnames(tmp) <- name[!name %in% c("", "#")]
        nominal.track <- cbind(x = tmp$ox, y = tmp$oy)
        pred.track <- cbind(x = tmp$px, y = tmp$py)
        most.prob.track <- cbind(x = tmp$smoothX, y = tmp$smoothY)
        SST <- cbind(o = tmp$oSST, p = tmp$pSST, smooth = tmp$smoothSST)
        days.at.liberty <- cumsum(tmp$dt)
        date <- matrix(as.numeric(unlist(strsplit(as.character(tmp$date), 
            "/"))), ncol = 3, byrow = TRUE)
        colnames(date) <- c("year", "month", "day")
        var.most.prob.track <- cbind(tmp$Psmooth11, tmp$Psmooth12, 
            tmp$Psmooth21, tmp$Psmooth22)
    }
    return(list(npar = npar, nlogL = nlogL, max.grad.comp = max.grad.comp, 
        estimates = estimates, std.dev = std.dev, nominal.track = nominal.track, 
        pred.track = pred.track, most.prob.track = most.prob.track, 
        var.most.prob.track = var.most.prob.track, days.at.liberty = days.at.liberty, 
        date = date, spd = spd, hdg = hdg, SST = SST))
}
".sys" <-
function (cmd) 
if (.Platform$OS.type == "windows") {
    shell(cmd, invisible = TRUE)
} else {
    system(cmd)
}
